
# q-expansion of newform 2028.4.a.g, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2028
weight = 4
poly_data = [-3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [3, 0], [0, 2], [0, -3], [0, -5], [0, 0], [-18, 0], [0, 13], [-24, 0], [6, 0], [0, 31], [0, -54], [0, -32], [-20, 0], [0, 85], [-306, 0], [0, 203], [70, 0], [0, -131], [0, -297], [0, -4], [-416, 0], [0, -311], [0, -8], [0, 152], [-1278, 0], [-56, 0], [-876, 0], [0, 394], [690, 0], [-1744, 0], [-516, 0], [0, -456], [500, 0], [0, 482], [0, -179], [-586, 0], [0, 123], [0, -271], [-3222, 0], [-2220, 0], [1570, 0], [1968, 0], [0, 1328], [0, 754], [-3016, 0], [-5060, 0], [0, -401], [0, 57], [0, 994], [5478, 0], [0, -621], [0, 1584], [-3252, 0], [-4386, 0], [-5928, 0], [3702, 0], [0, -1279], [-8174, 0], [0, 368], [-820, 0], [0, -1010], [0, -2373], [7368, 0], [-326, 0], [0, 1942], [0, 697], [5342, 0], [9780, 0], [0, -522], [0, -1980], [0, 1505], [-6064, 0], [-4418, 0], [0, 633], [0, -2525], [13122, 0], [0, -2770], [0, 2436], [0, -3924], [-12708, 0], [0, -498], [0, -2661], [-9682, 0], [4216, 0], [-8508, 0], [0, 2636], [0, 740], [0, 414], [0, 2417], [5316, 0], [0, 5237], [0, 2795], [-8676, 0], [0, -5451], [9576, 0], [0, 118], [-9222, 0], [-5068, 0], [0, -3426], [-3076, 0], [0, -5758], [12756, 0], [-11754, 0], [-13396, 0], [0, 2144], [0, 7053], [0, -2348], [-7800, 0], [14362, 0], [-6880, 0], [0, 6886], [0, 7536], [0, 6311], [0, -6195], [19902, 0], [0, -4083], [-10392, 0], [21174, 0], [24828, 0], [0, 4594], [6382, 0], [1854, 0], [0, -2693], [0, -4939], [24762, 0], [0, 5810], [36672, 0], [-28696, 0], [0, -6910], [0, 1891], [0, 2055], [33808, 0], [29230, 0], [0, 6672], [0, -4576], [0, -9990], [0, 4907], [35034, 0], [25338, 0], [0, 8321], [0, -8126], [664, 0], [0, -11773], [24122, 0], [0, -1767], [0, -718], [32790, 0], [-2444, 0], [0, -2605], [0, -4590], [18222, 0], [-32668, 0], [14424, 0], [23996, 0], [-19968, 0], [2488, 0], [0, -3084], [51898, 0], [0, -15354], [0, -4969], [29622, 0], [0, 12059], [22980, 0], [0, 4564], [0, 7041], [26336, 0], [-37282, 0], [0, -12848], [51762, 0], [0, 4925], [0, 5574], [42360, 0], [0, 13812], [-31088, 0], [18474, 0], [0, -5809], [0, 11454], [36136, 0], [14758, 0], [0, 12393], [11748, 0], [19102, 0], [0, 4040], [0, 14549], [60786, 0], [19690, 0], [0, -16059], [0, 7660], [0, 19981], [-30958, 0], [0, 8333], [-49108, 0], [0, 15578], [3108, 0], [-13626, 0], [0, 8984], [21962, 0], [0, -6580], [-37752, 0], [0, -17766], [-1072, 0], [0, -6106], [34802, 0], [0, -16877], [46758, 0], [0, -4433], [24252, 0], [0, 22536], [-55300, 0], [-8818, 0], [-7506, 0], [2584, 0], [0, -9805], [0, 6863], [0, -9452], [-24032, 0], [19794, 0], [0, -4897], [0, -14102], [-10018, 0], [0, -20619], [0, 20428], [0, -24697], [-95196, 0], [-686, 0], [29994, 0], [-7296, 0], [1400, 0], [0, -4301], [-88070, 0], [-82996, 0], [0, 23255], [35814, 0], [-50764, 0], [0, -13453], [0, -464], [0, -20122], [-57444, 0], [-65208, 0], [0, -4887], [-19348, 0], [47656, 0], [0, -16222], [0, 8900], [-27576, 0], [0, 13273], [0, 24495], [0, 12817], [-15264, 0], [0, 12270], [0, -28140], [0, 1609], [-24778, 0], [-24138, 0], [0, 11583], [10942, 0], [0, -4519], [84738, 0], [0, -38236], [-1456, 0], [-95124, 0], [0, 4078], [-60538, 0], [0, 27676], [63532, 0], [0, -5394], [0, -16104], [0, -32081], [-55998, 0], [97034, 0], [0, -23211], [0, -6428], [-98912, 0], [-66878, 0], [0, 24875], [0, 27869], [0, -4042], [0, -908], [-53340, 0], [-106128, 0], [0, -40723], [100536, 0], [0, 22006], [0, -33785], [-35664, 0], [16306, 0], [0, -10766], [0, -9187], [-132930, 0], [-3018, 0], [-93444, 0], [0, 28720], [0, -7837], [79430, 0], [-12006, 0], [-9376, 0], [10674, 0], [-96012, 0], [0, 38371], [-98714, 0], [0, -18662], [16720, 0], [79452, 0], [159428, 0], [0, -43288], [27756, 0], [-33226, 0], [0, -3583], [81938, 0], [-84912, 0], [0, 14474], [-50046, 0], [-107156, 0], [0, -20575], [-76502, 0], [0, 1233], [0, -18589], [0, -20936], [113358, 0], [9172, 0], [0, -38252], [52230, 0], [0, 49553], [0, -21472], [-8942, 0], [0, 16059], [0, -21183], [-65664, 0], [58110, 0], [0, -22906], [5286, 0], [-2000, 0], [0, 4631], [0, -29679], [0, 53269], [0, 9254], [0, 23764], [0, -23308], [94288, 0], [0, 40750], [-90006, 0], [0, -21010], [115240, 0], [0, 54918], [-123708, 0], [-36434, 0], [0, 39071], [0, -23003], [14130, 0], [0, 44685], [0, -24260], [0, 4006], [178688, 0], [-114190, 0], [15402, 0], [0, -39491], [0, -7611], [-223698, 0], [0, 1735], [0, 23798], [222150, 0], [72624, 0], [0, 29261], [-45884, 0], [56810, 0], [0, -28518], [0, 53749], [-99386, 0], [-136356, 0], [86092, 0], [0, -11883], [-95826, 0], [132280, 0], [-33946, 0], [0, -2855], [48096, 0], [0, 53776], [-158190, 0], [156742, 0], [0, 42394], [0, 21736], [0, 11733], [0, -40884], [0, 12059], [0, -8679], [0, -30465], [101138, 0], [0, -11719], [-265056, 0], [0, 22232], [0, -65750], [0, 78971], [251356, 0], [0, -33079], [-110198, 0], [0, 17607], [-281658, 0], [-96572, 0], [0, 61950], [0, -17302], [271230, 0], [0, 2657], [0, 8728], [0, 45206], [145000, 0], [0, -12950], [0, -61036], [0, -33605], [0, 35959], [-251378, 0], [159246, 0], [-203148, 0], [177604, 0], [-51386, 0], [-190698, 0], [0, 44651], [188872, 0], [0, 45124], [4872, 0], [184794, 0], [0, 50630], [0, 59747], [179940, 0], [0, 12748], [0, 26774], [-91356, 0], [0, 608], [0, -85977], [-268968, 0]]
